@@ -3,8 +3,8 @@
 ;; Copyright (C) 2019 - 2023 Jared Lumpe
 
 ;; Author: Jared Lumpe <jared@jaredlumpe.com>
-;; Package-Version: 0.3.0
-;; Package-Revision: 57a43e3b3e40
+;; Package-Version: 20250825.125
+;; Package-Revision: 0f7c63b9bbbf
 ;; Keywords: outlines
 ;; Homepage: https://github.com/jlumpe/ox-json
 
@@ -62,6 +62,9 @@
 ;;   in the :json-property-types option. If true these properties will be exported
 ;;   using `ox-json-encode-auto'.
 
+;; :json-postprocess (symbol) - How to postprocess the final output. Values are `pretty'
+;;   (indent properly), `minimal' (remove whitespace), and nil (nothing, maybe faster?).
+
 ;;; Code:
 
 (eval-when-compile (require 'cl-lib))
@@ -96,7 +99,7 @@
   "Default exporter function for each element property type.
 
 Plist mapping property symbols in
-`ox-json-default-property-types' to exporter function. These can
+`ox-json-default-property-types' to exporter functions. These can
 be overridden with the :json-exporters option.")
 
 (defconst ox-json-default-property-types
@@ -104,13 +107,16 @@ be overridden with the :json-exporters option.")
     all (
       ; Never include parent, leads to infinite recursion
       :parent nil
+      :buffer nil
       ; These properties have to do with absolute buffer positions and thus probably aren't useful to export
       :begin nil
       :end nil
       :contents-begin nil
       :contents-end nil
+      :robust-begin nil
+      :robust-end nil
+      :post-affiliated nil
       ; These can be useful when converting from JSON to another format
-      :post-affiliated number
       :pre-blank number
       :post-blank number)
     babel (
@@ -294,6 +300,14 @@ These can be overridden with the :json-property-types option.")
 (defgroup ox-json nil "Customization for the ox-json package" :group 'outline)
 
 
+;;; Private globals
+
+(defvar ox-json--current-node nil
+  "The current org node being transcoded, or a string indicating the current export context.
+
+Used for error reporting.")
+
+
 ;;; Generic utility code
 
 (defun ox-json--merge-alists (&rest alists)
@@ -344,9 +358,29 @@ These can be overridden with the :json-property-types option.")
 ;;; Org-mode utility code
 
 (defun ox-json-node-properties (node)
-  "Get property plist of element/object NODE."
-  ; It's the 2nd element of the list
-  (cadr node))
+  "Org v9.7 introduced two significant changes to the AST that must be
+considered when enumerating a node's properties:
+
+     1. Some properties which were previously present in the property
+     list (e.g. :begin and :end) are now stored as elements of a vector
+     under the :standard-properties key in the property list.
+
+     2. Property values can now be 'deferred', meaning they are not
+     calculated until accessed via a getter function like
+     ~org-element-property~.
+
+~org-element-properties-map~ is now the recommended way to traverse a
+node's properties and handles both of these changes."
+  (if (fboundp 'org-element-properties-map)
+    (let ((expanded-properties nil))
+      (org-element-properties-map
+       (lambda (name value)
+         (setq expanded-properties (plist-put expanded-properties name value)))
+       node t)
+      expanded-properties)
+    ; for org versions < 9.7, just return the property list, which is the second
+    ; element of the list
+    (cadr node)))
 
 (defun ox-json--is-node (value)
   "Check if VALUE is an org element/object."
@@ -376,6 +410,40 @@ ZONE is a time zone to pass to `format-time-string'."
       (year
         (format-time-string "%Y-%m-%d" (encode-time 0 0 0 day month year zone) zone)))))
 
+(defun ox-json--is-drawer-property-name (name &optional _info)
+  "Try to determine if a headline property name came from a property drawer.
+
+NAME is the property name as symbol or string."
+  (when (symbolp name)
+    (setq name (symbol-name name)))
+  (s-uppercase-p name))
+
+(defun ox-json--separate-drawer-properties (properties info)
+  "Separate drawer properties from a headline's property plist.
+
+PROPERTIES is a plist of the headline's properties, as from
+`ox-json-node-properties'.
+
+INFO is the plist of export options.
+
+Returns a cons cell containing two plists, the regular properties in the car
+and the drawer properties in the cdr."
+  (let ((regular-props nil)
+        (drawer-props nil))
+    (ox-json--loop-plist (name value properties)
+      do (if (ox-json--is-drawer-property-name name info)
+           ; Property drawer
+           (let* ((realname (intern (s-replace "+" "" (symbol-name name))))
+                  (existing (plist-get drawer-props realname))
+                  (strval (format "%s" value))
+                  (newval (if existing
+                             (concat existing " " strval)
+                             strval)))
+             (setq drawer-props (plist-put drawer-props realname newval)))
+           ; Regular property
+           (setq regular-props (plist-put regular-props name value))))
+    (cons regular-props drawer-props)))
+
 
 ;;; Define the backend
 
@@ -393,20 +461,29 @@ ZONE is a time zone to pass to `format-time-string'."
       for type in (append org-element-all-elements org-element-all-objects)
       collect (cons type #'ox-json-transcode-base)))
   ;; Filters
-  :filters-alist '()
+  :filters-alist `(
+    (:filter-final-output . ,#'ox-json-filter-final-output)
+  )
   ;; Options
   :options-alist
   '(
+     ; Property name specifying data type of exported JSON object
      (:json-data-type-property nil "json-data-type-property" "$$data_type")
+     ; Overrides to ox-json-default-type-exporters
      (:json-exporters nil nil nil)
+     ; Overrides to ox-json-default-property-types
      (:json-property-types nil nil nil)
+     ; If non-nil abort on encountering errors, otherwise encode errors in output JSON
      (:json-strict nil nil nil)
-     (:json-include-extra-properties nil nil t))
+     ; Include properties not defined in ox-json-default-property-types
+     (:json-include-extra-properties nil nil t)
+     ; How to post-process the final output
+     (:json-postprocess nil nil 'pretty))
   ;; Menu
   :menu-entry
   '(?j "Export to JSON" (
-	(?J "As JSON buffer" ox-json-export-to-buffer)
-	(?j "To JSON file" ox-json-export-to-file))))
+	  (?J "As JSON buffer" ox-json-export-to-buffer)
+	  (?j "To JSON file" ox-json-export-to-file))))
 
 
 ;;; User export functions
@@ -484,9 +561,37 @@ Code copied from `org-export-as'."
 
 INFO is the plist of export options.
 MSG is the error message.
-ARGS are objects to insert into MSG using `format'."
+ARGS are objects to insert into MSG using `format-message'."
   (ox-json-make-object "error" info
-    `((message string ,(apply #'format msg args)))))
+    `((message string ,(apply #'format-message msg args)))))
+
+(defun ox-json--signal-error (msg &rest args)
+  "Signal an error, adding information on the current element being transcoded if applicable.
+
+MSG is the error message.
+ARGS are objects to insert into MSG using `format-message'."
+  (let* ((msg-formatted (apply #'format-message msg args))
+         (node-type (org-element-type ox-json--current-node))
+         (context-string))
+    (setq context-string
+      (cond
+        ; Is an actual org node - give type and location
+        (node-type
+          (format "%s element on line %s"
+            node-type
+            (line-number-at-pos (org-element-property :begin ox-json--current-node))
+          ))
+        ; A string - use directly for context
+        ((stringp ox-json--current-node)
+          ox-json--current-node)
+        ; Invalid?
+        (ox-json--current-node
+          "<unknown>")
+        ; Otherwise nil
+      ))
+    (when context-string
+      (setq msg-formatted (format "Error transcoding %s: %s" context-string msg-formatted)))
+    (error msg-formatted)))
 
 (defun ox-json--error (info msg &rest args)
   "Either signal an error or return an encoded error object.
@@ -497,7 +602,7 @@ INFO is the plist of export options.
 MSG is the error message.
 ARGS are objects to insert into MSG using `format'."
   (if (plist-get info :json-strict)
-    (apply #'error msg args)
+    (apply #'ox-json--signal-error msg args)
     (ox-json--make-error-obj info msg args)))
 
 (cl-defun ox-json--type-error (type value info &optional (maxlen 200))
@@ -519,7 +624,7 @@ MAXLEN is the number of characters to truncate the representation of VALUE at."
 
 ;;; Encoders for generic data types
 
-(cl-defun ox-json-encode-bool (value &optional info strict)
+(defun ox-json-encode-bool (value &optional info strict)
   "Encode VALUE to JSON as boolean.
 
 INFO is the plist of export options.
@@ -623,7 +728,7 @@ empty array, false, or null. A null value is arbitrarily returned in this case."
         (ox-json-export-data value info)
         (ox-json-encode-array value info)))
     (t
-      (ox-json--error info "Don't know how to encode value %S"  value))))
+      (ox-json--error info "Can't automatically encode value of type %S" (type-of value)))))
 
 (defun ox-json--get-type-encoder (typekey &optional info)
   "Get the type encoder function for type symbol TYPEKEY.
@@ -862,6 +967,12 @@ JSON-encoded values."
       info
       type-plists)))
 
+(defun ox-json--skip-property (property)
+  "Return non-nil if an element property PROPERTY should be skipped, regardless of the value of the
+:json-property-types option."
+  ; Apparently 9.7 introduces some private property names, skip these.
+  (cl-search "--" (symbol-name property)))
+
 (defun ox-json--export-properties-base (property-plist default-type info &rest type-plists)
   "Export org node property values by looking up their types in a series of plists.
 
@@ -879,7 +990,7 @@ JSON-encoded values."
     (ox-json--loop-plist (key value property-plist)
       do (setq property-type
            (apply #'ox-json--plists-get-default key default-type type-plists))
-      if property-type
+      if (and property-type (not (ox-json--skip-property key)))
         collect (cons key (ox-json-encode-with-type property-type value info)))))
 
 (cl-defun ox-json-export-node-base
@@ -892,7 +1003,7 @@ JSON-encoded values."
     (contents (ox-json-export-contents node info)))
   "Base export function for a generic org element/object.
 
-NODE is an org element or object and INFO is the export environment plist.
+NODE is an org element or object.
 INFO is the plist of export options.
 PROPERTY-TYPES is a plist of type symbols which override the default way of
 determining how to encode property values (see equivalent argument in
@@ -938,14 +1049,15 @@ INFO is the plist of export options."
   (unless (string= text "")
     (json-encode-string text)))
 
-(cl-defun ox-json-transcode-base (node _contents info)
+(defun ox-json-transcode-base (node _contents info)
   "Default transcoding function for all element/object types.
 
 NODE is an element or object to encode.
 CONTENTS is a string containing the encoded contents of the node,
 but its value is ignored (`ox-json-export-contents' is used instead).
 INFO is the plist of export options."
-  (ox-json-export-node-base node info))
+  (let ((ox-json--current-node node))
+    (ox-json-export-node-base node info)))
 
 (defun ox-json-document-properties (info)
   "Get alist of top level document properties (values already encoded).
@@ -969,7 +1081,8 @@ INFO is the plist of export options."
 CONTENTS is a string containing the encoded document contents,
 but its value is ignored (`ox-json-export-contents' is used instead).
 INFO is the plist of export options."
-  (let* ((properties (ox-json-document-properties info))
+  (let* ((ox-json--current-node "<document root>")
+         (properties (ox-json-document-properties info))
          (properties-encoded (ox-json-encode-alist-raw nil properties info))
          (parse-tree (plist-get info :parse-tree))
          (contents-encoded (ox-json-export-contents parse-tree info)))
@@ -980,40 +1093,6 @@ INFO is the plist of export options."
          (contents . ,contents-encoded))
       info)))
 
-(defun ox-json--is-drawer-property-name (name &optional _info)
-  "Try to determine if a headline property name came from a property drawer.
-
-NAME is the property name as symbol or string."
-  (when (symbolp name)
-    (setq name (symbol-name name)))
-  (s-uppercase-p name))
-
-(defun ox-json--separate-drawer-properties (properties info)
-  "Separate drawer properties from a headline's property plist.
-
-PROPERTIES is a plist of the headline's properties, as from
-`ox-json-node-properties'.
-
-INFO is the plist of export options.
-
-Returns a cons cell containing two plists, the regular properties in the car
-and the drawer properties in the cdr."
-  (let ((regular-props nil)
-        (drawer-props nil))
-    (ox-json--loop-plist (name value properties)
-      do (if (ox-json--is-drawer-property-name name info)
-           ; Property drawer
-           (let* ((realname (intern (s-replace "+" "" (symbol-name name))))
-                  (existing (plist-get drawer-props realname))
-                  (strval (format "%s" value))
-                  (newval (if existing
-                             (concat existing " " strval)
-                             strval)))
-             (setq drawer-props (plist-put drawer-props realname newval)))
-           ; Regular property
-           (setq regular-props (plist-put regular-props name value))))
-    (cons regular-props drawer-props)))
-
 (cl-defun ox-json-transcode-headline (headline _contents info &rest kw &key extra)
   "Transcode a headline element to JSON.
 
@@ -1025,7 +1104,8 @@ KW is a plist of keyword arguments to pass to `ox-json-export-node-base'.
 EXTRA is an alist of additional properties to attach to the exported JSON object
 at the top level."
   (pcase-let*
-    ((all-props (ox-json-node-properties headline))
+    ((ox-json--current-node headline)
+     (all-props (ox-json-node-properties headline))
      (`(,regular-props . ,drawer-props)
        (ox-json--separate-drawer-properties all-props info))
      (regular-encoded
@@ -1083,8 +1163,9 @@ LINK is the parsed link to transcode.
 CONTENTS is a string containing the encoded contents of the element,
 but its value is ignored (`ox-json-export-contents' is used instead).
 INFO is the plist of export options."
-  (ox-json-export-node-base link info
-    :extra-properties (ox-json-link-extra-properties link info)))
+  (let ((ox-json--current-node link))
+    (ox-json-export-node-base link info
+      :extra-properties (ox-json-link-extra-properties link info))))
 
 (defun ox-json-timestamp-extra-properties (timestamp info)
   "Get additional properties to export from a timestamp object.
@@ -1104,8 +1185,38 @@ TIMESTAMP is the parsed link to transcode.
 CONTENTS is a string containing the encoded contents of the element,
 but its value is ignored (`ox-json-export-contents' is used instead).
 INFO is the plist of export options."
-  (ox-json-export-node-base timestamp info
-    :extra-properties (ox-json-timestamp-extra-properties timestamp info)))
+  (let ((ox-json--current-node timestamp))
+    (ox-json-export-node-base timestamp info
+      :extra-properties (ox-json-timestamp-extra-properties timestamp info))))
+
+
+;;; Filter functions
+
+(defun ox-json-filter-final-output (text back-end info)
+  "Post-process the entire output."
+  (let ((postprocess (plist-get info :json-postprocess)))
+    (cond
+      ; Not JSON?
+      ((not (eq back-end 'json))
+        text)
+      ; Post-process
+      ((memq postprocess '(pretty minimal))
+        ; Is this always available?
+        (if (fboundp 'json-pretty-print-buffer)
+          (with-temp-buffer
+            (insert text)
+            (json-pretty-print-buffer (eq postprocess 'minimal))
+            (buffer-string))
+          (warn "Unable to post-process JSON, json-pretty-print-buffer not available.")
+          text))
+      ; No post-processing
+      ((not postprocess)
+        text)
+      ; Invalid
+      (t
+        (warn "Invalid value for :json-postprocess option: %S" postprocess)
+        text)
+    )))
 
 
 (provide 'ox-json)
